#include <ansi_c.h>
#include <formatio.h>
#include <utility.h>
#include <visa.h>
#include "rsnrvd.h"

#define rsNRVD_REVISION     "Rev 1.3, 1/2009, CVI 8.5" /*  Instrument driver revision */
#define BUFFER_SIZE         512L         /* File I/O buffer size */
    
/*===========================================================================*/
/*= ROHDE&SCHWARZ POWER METER NRVD ==========================================*/
/* LabWindows/CVI 3.1 Instrument Driver                                      */
/* Original Release: January 1996                                            */
/* By: Jan Sima, Technical University Ostrava, Czech Republic                */
/*     PH. +420 69 699 6146  Fax +420 69 691 9597  e-mail Jan.Sima@vsb.cz    */
/*                                                                           */
/* For LabWindows Technical Support in the United States call:               */
/*              National Instruments, Austin, Texas                          */
/*              Ph (800) 433-3488    Fax (512) 794-5678                      */
/*                                                                           */
/* Originally written in C                                                   */
/* Modification History:                                                     */
/* Modifications:        Added switch to BUS trigger mode in rsNRVD_trigger  */
/*                       SWR is now ViReal64 in rsNRVD_configMode            */
/*                       and other cosmetic changes                          */
/*                                                                           */
/* Modification Date:    June 15, 1999                                       */
/*                                                                           */
/* Modified by:          Arnd Diestelhorst, Rohde & Schwarz                  */
/*                       Arnd.Diestelhorst@rsd.rsd.de                        */
/* Modifications:        Added _VI_FAR for MDLL Compatibility                */
/*                                                                           */
/* Modification Date:    June 26, 1996                                       */
/*                                                                           */
/* Modified by:          DS, National Instruments, Austin, Texas             */
/*                       Phone (800) 433-3488  Fax (512) 794-5678            */
/*                                                                           */
/* Modifications:        1. removed bug in function Measure                  */
/*                       2. removed GPIB address bug from UIR                */
/*                                                                           */
/* Modification Date:    May 5, 1999 (Rev 1.1, 05/1999, CVI 5.0.1)           */                          
/*                                                                           */
/* Modified by:          Martin Koutny, ELCOM a.s. (CZ),                     */
/* E-mail:               martin.koutny@vsb.cz                                */
/*                                                                           */
/* Modifications:        Renamed utility functions and other minor fixes     */
/*                                                                           */
/* Modification Date:    June 5, 1999 (Rev 1.2, 06/1999, CVI 5.0.1)          */                          
/* Modified by:          Jan Sima, ELCOM a.s. (CZ),                          */
/* E-mail:               jan.sima@vsb.cz                                     */
/*                                                                           */
/* Modifications:        Added switch to BUS trigger mode in rsNRVD_trigger  */
/*                       SWR is now ViReal64 in rsNRVD_configMode            */
/*                       and other cosmetic changes                          */
/* Modification Date:    June 15, 1999                                       */
/*                                                                           */
/* Modified by:          Arnd Diestelhorst, Rohde & Schwarz                  */
/*                       Arnd.Diestelhorst@rsd.rsd.de                        */
/*****************************************************************************/

 
/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/
static ViString chanSel[2] = {":INP:NSEL 2",":INP:NSEL 1"};
static ViString rangSel[3] = {"%s[a]<;:AMPL:RANG:AUTO OFF;:AMPL:RANG %f%s","%s[a]<;:AMPL:RANG:AUTO ON","%s[a]<;:AMPL:RANG:AUTO ONCE"};
static ViString rangUnitSel[10] = {"V","MV","UV","W","MW","UW","NW","DBM","DBV","DBUV"};
static ViString funcSel[29] = {";:AMPL:UNIT V;",";:AMPL:UNIT W;",";:AMPL:UNIT DBV;",";:AMPL:UNIT DBM;",";:AMPL:UNIT DBUV;",
                               ";:VOLT:UNIT LIN;",";:POW:UNIT LIN;",";:VOLT:UNIT PCT;",";:POW:UNIT PCT;",";:VOLT:UNIT DB;",
                               ";:POW:UNIT DB;",";:VOLT:UNIT REL;",";:POW:UNIT REL;",";:AMPL:REF:MVAL;:VOLT:UNIT LIN;",";:AMPL:REF:MVAL;:POW:UNIT LIN;",
                               ";:AMPL:REF:MVAL;:VOLT:UNIT PCT;",";:AMPL:REF:MVAL;:POW:UNIT PCT;",";:AMPL:REF:MVAL;:VOLT:UNIT DB;",";:AMPL:REF:MVAL;:POW:UNIT DB;",
                               ";:AMPL:REF:MVAL;:VOLT:UNIT REL;",";:AMPL:REF:MVAL;:POW:UNIT REL;",";:VOLT:UNIT XLIN;",";:POW:UNIT XLIN;",
                               ";:VOLT:UNIT XPCT;",";:POW:UNIT XPCT;",";:VOLT:UNIT XDB;",";:POW:UNIT XDB;",";:VOLT:UNIT XREL;",";:POW:UNIT XREL;"};
static ViString refValUnitSel[5] = {"V;","W;","DBV;","DBM;","DBUV;"};
static ViString measModeSel[6] = {";:FUNC \"POW:AC\"","%s[a]<;:FUNC \"POW:PULS %f\"",";:FUNC \"RFL\"",";:FUNC \"SWR\"",";:FUNC \"RTL\"",";:FUNC \"AM\""};
static ViString dispModeSel[5] = {";:DISP OFF",";:DISP ON;:DISP:ANN:AMPL SINGLE",";:DISP ON;:DISP:ANN:AMPL DUAL",";:DISP ON;:DISP:ANN:UNC:STAT ON;GSWR",";:DISP ON;:DISP:ANN:FREQ ON"};
static ViString analogDispSel[5] = {":DISP:ANN:BARG:STAT ON;AUTO ON;",":DISP:ANN:BARG:STAT ON;AUTO ONCE;",":DISP:ANN:BARG:STAT ON;:DISP:ANN:BARG:VOL;",":DISP:ANN:BARG:STAT ON;SCAL ON;SCAL",":DISP:ANN:BARG OFF;"};
static ViString extrDispSel[4] = {":CALC:EXTR OFF;:DISP:ANN:AMPL:EXTR OFF",":DISP:ANN:AMPL:EXTR ON;EXTR MIN",":DISP:ANN:AMPL:EXTR ON;EXTR MAX",":DISP:ANN:AMPL:EXTR ON;EXTR MAXM"};
static ViString freqCorrSel[3] = {";:SENS:CORR:FREF",";:SENS:FREQ:STAT ON;:SENS:CORR:FREF:STAT ON;",";:SENS:CORR:FREF:STAT OFF"};
static ViString OnOffSel[2] = {" OFF"," ON"};
static ViString zeroCorrSel[2] = {";:CORR:ZERO OFF",";:CORR:ZERO:INIT?"};
static ViString trigModeSel[6] = {"*CLS;:TRIG:SOUR IMM","*CLS;*TRG","*CLS;:TRIG:SOUR EXT",":CALC:EXTR:INIT",":CALC:EXTR OFF","*CLS;:TRIG:SOUR BUS"};

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* rsNRVD_stringValPair is used in the rsNRVD_errorMessage function          */
/* rsNRVD_statusDataRanges is used to track session dependent status & ranges*/
/*===========================================================================*/
typedef struct  rsNRVD_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  rsNRVD_tStringValPair;
 
/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean rsNRVD_invalidViBooleanRange (ViBoolean val);
ViBoolean rsNRVD_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean rsNRVD_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus rsNRVD_initCleanUp (ViSession openRMSession, ViPSession openInstrSession, ViStatus currentStatus);
ViStatus rsNRVD_instrStatus (ViSession instrSession);
ViStatus rsNRVD_defaultInstrSetup (ViSession openInstrSession);

/*****************************************************************************/
/*------ INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINE DECLARATIONS HERE ------*/
/*****************************************************************************/

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_init (ViRsrc resourceName, ViBoolean IDQuery,
                    ViBoolean resetDev, ViPSession instrSession)
{
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViSession rmSession = 0;
    ViUInt32 retCnt = 0;
    ViByte rdBuffer[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/
    if (rsNRVD_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (rsNRVD_invalidViBooleanRange (resetDev))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/
    if ((rsNRVD_status = viOpenDefaultRM (&rmSession)) < 0)
        return rsNRVD_status;

    if ((rsNRVD_status = viOpen (rmSession, resourceName, VI_NULL, VI_NULL, instrSession)) < 0) {
        viClose (rmSession);
        return rsNRVD_status;
    }

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    if ((rsNRVD_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 40000)) < 0)
            return rsNRVD_initCleanUp (rmSession, instrSession, rsNRVD_status);
    if ((rsNRVD_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return rsNRVD_initCleanUp (rmSession, instrSession, rsNRVD_status);
    if ((rsNRVD_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return rsNRVD_initCleanUp (rmSession, instrSession, rsNRVD_status);
    if ((rsNRVD_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return rsNRVD_initCleanUp (rmSession, instrSession, rsNRVD_status);

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery) {
        if ((rsNRVD_status = viWrite (*instrSession, "*IDN?", 5, &retCnt)) < 0)
            return rsNRVD_initCleanUp (rmSession, instrSession, rsNRVD_status);
        if ((rsNRVD_status = viRead (*instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
            return rsNRVD_initCleanUp (rmSession, instrSession, rsNRVD_status);

        Scan (rdBuffer, "ROHDE&SCHWARZ,NRVD,");
        if (NumFmtdBytes () != 19) 
            return rsNRVD_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
    }

    /*- Reset instrument ----------------------------------------------------*/
    if (resetDev) {
        if ((rsNRVD_status = rsNRVD_reset (*instrSession)) < 0)
            return rsNRVD_initCleanUp (rmSession, instrSession, rsNRVD_status);
    }       
    else  /*- Send Default Instrument Setup ---------------------------------*/
        if ((rsNRVD_status = rsNRVD_defaultInstrSetup (*instrSession)) < 0)
            return rsNRVD_initCleanUp (rmSession, instrSession, rsNRVD_status);
        
    rsNRVD_status = rsNRVD_instrStatus (*instrSession);    
    return rsNRVD_status;
}

/*===========================================================================*/
/* Function: Example                                                         */
/* Purpose:  This function is an example that shows how to use functions of  */
/*           this driver. It resets the instrument, sets measured quantity   */
/*           and performs asynchronous acquisition.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_example (ViSession instrSession,
                                  ViBoolean chan, 
                                  ViInt16 func,
                                  ViReal64 refValue,
                                  ViInt16 refValUnit,
                                  ViReal64 _VI_FAR measData[],
                                  ViInt16 numOfPoints)
{                                  
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViInt16 i = 0;
    ViBoolean cmpl = VI_FALSE;
    ViInt32 tmpI;
    ViChar tmpS[1024];
    ViReal64 measVal;

    if (rsNRVD_invalidViBooleanRange (chan))
        return VI_ERROR_PARAMETER2;
    if (rsNRVD_invalidViInt16Range (func, 0, 28))
        return VI_ERROR_PARAMETER3;
    if (rsNRVD_invalidViReal64Range (refValue, -1.0e9, 1.0e9))
        return VI_ERROR_PARAMETER4;     
    if (rsNRVD_invalidViReal64Range (refValUnit, 0, 4))
        return VI_ERROR_PARAMETER5;
    if (rsNRVD_invalidViInt16Range (numOfPoints, 1, 32767))
        return VI_ERROR_PARAMETER7;

    if ((rsNRVD_status = rsNRVD_reset (instrSession)) < 0)
        return rsNRVD_status;
        
    if ((rsNRVD_status = rsNRVD_trigger (instrSession, 1)) < 0)
        return rsNRVD_status;

    // To make sure there is always a current value available
    // make an extra measurement cycle
    
    if (func == 13 || func == 15 || func == 17 || func == 19) {
        if ((rsNRVD_status = rsNRVD_configFunction (instrSession, chan, 0, 0.0, 0)) < 0)
            return rsNRVD_status;
        if ((rsNRVD_status = rsNRVD_trigger (instrSession, 1)) < 0)
            return rsNRVD_status;
        cmpl = 0;   
        while (!cmpl) {
            if ((rsNRVD_status = rsNRVD_operComplete (instrSession, &cmpl)) < 0)
                return rsNRVD_status;
            Delay (0.2);       
        }   
        if ((rsNRVD_status = rsNRVD_readMeasurement (instrSession, chan, &measVal)) < 0)
            return rsNRVD_status;
    }

    if (func == 14 || func == 16 || func == 18 || func == 20) {
        if ((rsNRVD_status = rsNRVD_configFunction (instrSession, chan, 1, 0.0, 1)) < 0)
            return rsNRVD_status;
        if ((rsNRVD_status = rsNRVD_trigger (instrSession, 1)) < 0)
            return rsNRVD_status;
        cmpl = 0;   
        while (!cmpl) {
            if ((rsNRVD_status = rsNRVD_operComplete (instrSession, &cmpl)) < 0)
                return rsNRVD_status;
            Delay (0.2);       
        }   
        if ((rsNRVD_status = rsNRVD_readMeasurement (instrSession, chan, &measVal)) < 0)
            return rsNRVD_status;
    }

    if ((rsNRVD_status = rsNRVD_configFunction (instrSession, chan, func, refValue, refValUnit)) < 0)
        return rsNRVD_status;

    if ((rsNRVD_status = rsNRVD_error_query (instrSession, &tmpI, tmpS)) < 0)
        return rsNRVD_status;

    for (i = 0; i < numOfPoints; i++) {

         if ((rsNRVD_status = rsNRVD_trigger (instrSession, 1)) < 0)
             return rsNRVD_status;

         cmpl = 0;   
         while (!cmpl) {
            if ((rsNRVD_status = rsNRVD_operComplete (instrSession, &cmpl)) < 0)
                return rsNRVD_status;
            Delay (0.2);       
         }
         
         if ((rsNRVD_status = rsNRVD_readMeasurement (instrSession, chan, &measData[i])) < 0)
             return rsNRVD_status;
               
    }        

    rsNRVD_reset (instrSession);
         
    rsNRVD_status = rsNRVD_instrStatus (instrSession);     
    return rsNRVD_status;        
}

/*===========================================================================*/
/* Function: Configure Sensors                                               */
/* Purpose:  This function configures the sensor used by the instrument.     */
/*           It configures its ranging, sets the filter degree, input        */
/*           attenuation and reference impedance.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_configSensors (ViSession instrSession,
                                        ViBoolean channel,
                                        ViInt16 rangMode,
                                        ViInt16 filterDeg,
                                        ViReal64 manRange,
                                        ViInt16 rngUnit,
                                        ViReal64 attOnInput,
                                        ViReal64 refImp)
{
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte writeBuffer[BUFFER_SIZE];
    

    if (rsNRVD_invalidViBooleanRange (channel))
        return VI_ERROR_PARAMETER2;
    if (rsNRVD_invalidViInt16Range (rangMode, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (rsNRVD_invalidViInt16Range (filterDeg, 0, 13))
        return VI_ERROR_PARAMETER4;     
    if (rsNRVD_invalidViReal64Range (manRange, 0.0, 1.0e9))
        return VI_ERROR_PARAMETER5;
    if (rsNRVD_invalidViInt16Range (rngUnit, 0, 9))
        return VI_ERROR_PARAMETER6; 
    if (rsNRVD_invalidViReal64Range (attOnInput, -200, 200))
        return VI_ERROR_PARAMETER7; 
    if (rsNRVD_invalidViReal64Range (refImp, 0.0, 1000.0))
        return VI_ERROR_PARAMETER8; 
    
    Fmt (writeBuffer,"%s<%s", chanSel[channel]);
    if (!rangMode) 
        Fmt (writeBuffer, rangSel[0], manRange, rangUnitSel[rngUnit]);      
    else 
        Fmt (writeBuffer, rangSel[rangMode]);
    if (filterDeg == 13) 
        Fmt (writeBuffer,"%s[a]<;:CALC:FILT:AUTO ON");
    else 
        Fmt (writeBuffer,"%s[a]<;:CALC:FILT:AUTO OFF;:CALC:FILT:NSEL %d[b2]", filterDeg);
    if (refImp >= 1.0) {
        Fmt (writeBuffer,"%s[a]<;:INP:IMP %f", refImp);
    }
    Fmt (writeBuffer,"%s[a]<;:AMPL:ATT %f", attOnInput);
    
    if ((rsNRVD_status = viWrite (instrSession, writeBuffer, strlen (writeBuffer),
        &retCnt)) < 0)
        return rsNRVD_status;
    
    rsNRVD_status = rsNRVD_instrStatus (instrSession);     
    return rsNRVD_status;   
}
           
/*===========================================================================*/
/* Function: Configure Function                                              */
/* Purpose:  This function configures the measurement function of the        */
/*           instrument and specifies a reference value used for referenced  */
/*           measurements.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_configFunction (ViSession instrSession,
                                         ViBoolean channel,
                                         ViInt16 func,
                                         ViReal64 refValue,
                                         ViInt16 refValUnit)
{
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViUInt32 sndCnt;
    ViByte writeBuffer[BUFFER_SIZE];

    if (rsNRVD_invalidViBooleanRange (channel))
        return VI_ERROR_PARAMETER2;
    if (rsNRVD_invalidViInt16Range (func, 0, 28))
        return VI_ERROR_PARAMETER3;
    if (rsNRVD_invalidViReal64Range (refValue, -1.0e9, 1.0e9))
        return VI_ERROR_PARAMETER4;     
    if (rsNRVD_invalidViReal64Range (refValUnit, 0, 4))
        return VI_ERROR_PARAMETER5;
    
    Fmt (writeBuffer,"%s<%s", chanSel[channel]);
    sndCnt = NumFmtdBytes();    
    Fmt (writeBuffer,"%s[a]<%s", funcSel[func]);
    sndCnt += NumFmtdBytes();   
    if (func > 4 && func < 13) {
        Fmt (writeBuffer,"%s[a]<:AMPL:REF %f[e2]%s", refValue, refValUnitSel[refValUnit]);
        sndCnt += NumFmtdBytes();   
    }
    
    if ((rsNRVD_status = viWrite (instrSession, writeBuffer, sndCnt, &retCnt)) < 0)
        return rsNRVD_status;
        
    rsNRVD_status = rsNRVD_instrStatus (instrSession);     
    return rsNRVD_status;   
}

/*===========================================================================*/
/* Function: Configure Mode                                                  */
/* Purpose:  This function configures the measurement modes of the           */
/*           instrument.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_configMode (ViSession instrSession, 
                                     ViBoolean channel,
                                     ViInt16 measMode,
                                     ViReal64 dutyCycle)
{
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViUInt32 sndCnt;
    ViByte writeBuffer[BUFFER_SIZE];

    if (rsNRVD_invalidViBooleanRange (channel))
        return VI_ERROR_PARAMETER2;
    if (rsNRVD_invalidViInt16Range (measMode, 0, 5))
        return VI_ERROR_PARAMETER3;
    if (rsNRVD_invalidViReal64Range (dutyCycle, 0.0, 100.0))
        return VI_ERROR_PARAMETER4;     
    
    Fmt (writeBuffer,"%s<%s", chanSel[channel]);
    sndCnt = NumFmtdBytes();    
    if (measMode == 1)  
        Fmt (writeBuffer, measModeSel[1], dutyCycle);
    else 
        Fmt (writeBuffer,"%s[a]<%s", measModeSel[measMode]);
    sndCnt += NumFmtdBytes();   

    if ((rsNRVD_status = viWrite (instrSession, writeBuffer, sndCnt, &retCnt)) < 0)
        return rsNRVD_status;
    
    rsNRVD_status = rsNRVD_instrStatus (instrSession);     
    return rsNRVD_status;   
}

/*===========================================================================*/
/* Function: Configure Display                                               */
/* Purpose:  This function configures the digital display and the analog bar */
/*           display of the instrument.                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_configDisplay (ViSession instrSession,
                                        ViBoolean channel,
                                        ViInt16 dispRes,
                                        ViInt16 dispMode,
                                        ViInt16 analogBar,
                                        ViReal64 uppLim,
                                        ViReal64 lowLim,
                                        ViInt16 extrDisp,
                                        ViReal64 SWR)
{    
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViUInt32 sndCnt;
    ViByte writeBuffer[BUFFER_SIZE];

    if (rsNRVD_invalidViBooleanRange (channel))
        return VI_ERROR_PARAMETER2;
    if (rsNRVD_invalidViInt16Range (dispRes, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (rsNRVD_invalidViInt16Range (dispMode, 0, 4))
        return VI_ERROR_PARAMETER4;
    if (rsNRVD_invalidViInt16Range (analogBar, 0, 4))
        return VI_ERROR_PARAMETER5;
    if (rsNRVD_invalidViReal64Range (uppLim, -1.0e10, 1.0e10))
        return VI_ERROR_PARAMETER6;     
    if (rsNRVD_invalidViReal64Range (lowLim, -1.0e10, 1.0e10))
        return VI_ERROR_PARAMETER7;     
    if (rsNRVD_invalidViInt16Range (extrDisp, 0, 3))
        return VI_ERROR_PARAMETER8;
    if (rsNRVD_invalidViInt16Range (SWR, 1, 10))
        return VI_ERROR_PARAMETER9;
    
    Fmt (writeBuffer,"%s<%s", chanSel[channel]);
    sndCnt = NumFmtdBytes();    
    if (dispMode == 3) 
        Fmt (writeBuffer,"%s[a]<%s %f[e2];", dispModeSel[dispMode], SWR);
    else 
        Fmt (writeBuffer,"%s[a]<%s;", dispModeSel[dispMode]);
    sndCnt += NumFmtdBytes();   
    Fmt (writeBuffer,"%s[a]<:DISP:ANN:AMPL:NRES %d[b2];", dispRes + 3);
    sndCnt += NumFmtdBytes();   
    if (analogBar == 3) 
        Fmt (writeBuffer,"%s[a]<%s:LOW %f[e2];UPP %f[e2];", analogDispSel[analogBar], lowLim, uppLim);
    else 
        Fmt (writeBuffer,"%s[a]<%s", analogDispSel[analogBar]);
    sndCnt += NumFmtdBytes();   
    Fmt (writeBuffer,"%s[a]<%s", extrDispSel[extrDisp]);
    sndCnt += NumFmtdBytes();   
    
    if ((rsNRVD_status = viWrite (instrSession, writeBuffer, sndCnt, &retCnt)) < 0)
        return rsNRVD_status;
    
    rsNRVD_status = rsNRVD_instrStatus (instrSession);     
    return rsNRVD_status;   
}

/*===========================================================================*/
/* Function: Configure Frequency Correction                                  */
/* Purpose:  This function configures the frequency correction system of the */
/*           instrument.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_configFreqCorr (ViSession instrSession,
                                         ViBoolean channel,
                                         ViInt16 corrFreqInp,
                                         ViReal64 corrFreq,
                                         ViReal64 uppVolt,
                                         ViReal64 uppFreq,
                                         ViReal64 lowVolt,
                                         ViReal64 lowFreq)
{    
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViUInt32 sndCnt;
    ViByte writeBuffer[BUFFER_SIZE];

    if (rsNRVD_invalidViBooleanRange (channel))
        return VI_ERROR_PARAMETER2;
    if (rsNRVD_invalidViInt16Range (corrFreqInp, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (rsNRVD_invalidViReal64Range (corrFreq, 1.0e3, 9.99e11))
        return VI_ERROR_PARAMETER4;     
    if (rsNRVD_invalidViReal64Range (uppVolt, -12.0, 12.0))
        return VI_ERROR_PARAMETER5;     
    if (rsNRVD_invalidViReal64Range (uppFreq, 1.0e3, 9.99e11))
        return VI_ERROR_PARAMETER6;     
    if (rsNRVD_invalidViReal64Range (lowVolt, -12.0, 12.0))
        return VI_ERROR_PARAMETER7;     
    if (rsNRVD_invalidViReal64Range (lowFreq, 1.0e3, 9.99e11))
        return VI_ERROR_PARAMETER8;     
    
    Fmt (writeBuffer,"%s<%s", chanSel[channel]);
    sndCnt = NumFmtdBytes();    
    if (!corrFreqInp) 
        Fmt (writeBuffer,"%s[a]<%s %f[e2]", freqCorrSel[corrFreqInp], corrFreq);
    else 
        Fmt (writeBuffer,"%s[a]<%s", freqCorrSel[corrFreqInp]);
    sndCnt += NumFmtdBytes();   
    if (corrFreqInp == 1) {
        Fmt (writeBuffer,"%s[a]<:SENS:FREQ:ADJ:UPP %f[e2]V,%f[e2];LOW %f[e2]V,%f[e2]", 
                                                   uppVolt, uppFreq, lowVolt, lowFreq);
        sndCnt += NumFmtdBytes();   
    }                    
    
    if ((rsNRVD_status = viWrite (instrSession, writeBuffer, sndCnt, &retCnt)) < 0)
        return rsNRVD_status;
    
    rsNRVD_status = rsNRVD_instrStatus (instrSession);     
    return rsNRVD_status;   
}

/*===========================================================================*/
/* Function: Configure Options                                               */
/* Purpose:  This function the analog output options and the test generator  */
/*           of the instrument.                                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_configOptions (ViSession instrSession,
                                        ViInt16 anOutChanA,
                                        ViInt16 anOutChanB,
                                        ViBoolean testGen)
{    
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViUInt32 sndCnt;
    ViByte writeBuffer[BUFFER_SIZE];

    if (rsNRVD_invalidViInt16Range (anOutChanA, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (rsNRVD_invalidViInt16Range (anOutChanB, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (rsNRVD_invalidViBooleanRange (testGen))
        return VI_ERROR_PARAMETER4;
                                  
    Fmt (writeBuffer, "%s<:OUTP:ROSZ%s", OnOffSel[testGen]);
    sndCnt = NumFmtdBytes();    
    if (anOutChanA) {
        Fmt (writeBuffer, "%s[a]<;:OUTP1:DC%s", OnOffSel[--anOutChanA]);    
        sndCnt += NumFmtdBytes();   
    }   
    if (anOutChanB)  {
        Fmt (writeBuffer, "%s[a]<;:OUTP2:DC%s", OnOffSel[--anOutChanB]);    
        sndCnt += NumFmtdBytes();   
    }   
    if ((rsNRVD_status = viWrite (instrSession, writeBuffer, sndCnt, &retCnt)) < 0)
        return rsNRVD_status;
    
    rsNRVD_status = rsNRVD_instrStatus (instrSession);     
    return rsNRVD_status;   
}

/*===========================================================================*/
/* Function: Zero Correction                                                 */
/* Purpose:  This function turns On or Off the zero correction mode of the   */
/*           instrument.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_zeroCorrection (ViSession instrSession,
                                         ViBoolean channel,
                                         ViBoolean mode)
{
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViUInt32 sndCnt;
    ViByte wrRdBuffer[BUFFER_SIZE];
    ViInt16 zeroMeasResult = 0;

    if (rsNRVD_invalidViBooleanRange (channel))
        return VI_ERROR_PARAMETER2;
    if (rsNRVD_invalidViBooleanRange (mode))
        return VI_ERROR_PARAMETER3;
    
    Fmt (wrRdBuffer,"%s<%s%s", chanSel[channel], zeroCorrSel[mode]);
    sndCnt = NumFmtdBytes();    
    
    if ((rsNRVD_status = viWrite (instrSession, wrRdBuffer, sndCnt, &retCnt)) < 0)
        return rsNRVD_status;
        
    if (mode == VI_TRUE) {
        if ((rsNRVD_status = viRead (instrSession, wrRdBuffer, 256, &retCnt)) < 0)
            return rsNRVD_status;
        if (Scan (wrRdBuffer, "%s>%d[b2]", &zeroMeasResult) != 1)
            return VI_ERROR_INTERPRETING_RESPONSE;
        if (zeroMeasResult) 
            return VI_ERROR_ZERO_MEAS_FAILED;
    }
    
    rsNRVD_status = rsNRVD_instrStatus (instrSession);     
    return rsNRVD_status;   
}

/*===========================================================================*/
/* Function: Trigger                                                         */
/* Purpose:  This function controls the triggering system of the instrument. */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_trigger (ViSession instrSession,
                                  ViInt16 trigMode)
{
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViUInt32 sndCnt;
    ViByte writeBuffer[BUFFER_SIZE];

    if (rsNRVD_invalidViInt16Range (trigMode, 0, 5))
        return VI_ERROR_PARAMETER2;
    
    Fmt (writeBuffer, "%s<%s", trigModeSel[trigMode]);
    sndCnt = NumFmtdBytes();
    
    if ((rsNRVD_status = viWrite (instrSession, writeBuffer, sndCnt, &retCnt)) < 0)
        return rsNRVD_status;
    
    if (trigMode != 1) 
        rsNRVD_status = rsNRVD_instrStatus (instrSession);
    return rsNRVD_status;   
}

/*===========================================================================*/
/* Function: Operation Complete?                                             */
/* Purpose:  This function checks for the status of measurement.             */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_operComplete (ViSession instrSession,
                                       ViPBoolean complete)
{
    ViStatus rsNRVD_status = VI_SUCCESS;

    if ((rsNRVD_status = viReadSTB (instrSession, complete)) < 0)
         return rsNRVD_status;
    
    *complete = (*complete & 16) != 0;
         
    return rsNRVD_status;   
}

/*===========================================================================*/
/* Function: Measure                                                         */
/* Purpose:  This function performes a measurement and returns the results.  */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_measure (ViSession instrSession,
                                  ViPReal64 chanA,
                                  ViPReal64 chanB)
{
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte rdBuffer[BUFFER_SIZE];
                       
    if ((rsNRVD_status = viWrite (instrSession, ":MEAS?", 6, &retCnt)) < 0)
        return rsNRVD_status;
    
    if ((rsNRVD_status = viRead (instrSession, rdBuffer, 256, &retCnt)) < 0)
        return rsNRVD_status;
     
    if (Scan (rdBuffer, "%s>%f[p4];%f[p4]", chanA, chanB) != 2)
        return VI_ERROR_INTERPRETING_RESPONSE;
        
    if (*chanA >= 9.9e37 && *chanB >= 9.9e37)
        return VI_ERROR_MEASUREMENT_NOT_READY;
        
    rsNRVD_status = rsNRVD_instrStatus (instrSession);     
    return rsNRVD_status;   
}                                     

/*===========================================================================*/
/* Function: Read Measurement                                                */
/* Purpose:  This function reads the measurement from the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_readMeasurement (ViSession instrSession,
                                          ViBoolean chann,
                                          ViPReal64 result)
{
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViUInt32 sndCnt;
    ViByte wrRdBuffer[BUFFER_SIZE];

    if (rsNRVD_invalidViBooleanRange (chann))
        return VI_ERROR_PARAMETER2;
    
    Fmt (wrRdBuffer, "%s<:SENS%d[b2]:DATA:AMPL?", 2 - chann);
    sndCnt = NumFmtdBytes();
    
    if ((rsNRVD_status = viWrite (instrSession, wrRdBuffer, sndCnt, &retCnt)) < 0)
        return rsNRVD_status;
    
    if ((rsNRVD_status = viRead (instrSession, wrRdBuffer, 256, &retCnt)) < 0)
        return rsNRVD_status;
     
    if (Scan (wrRdBuffer, "%s>%f[p4]", result) != 1)
        return VI_ERROR_INTERPRETING_RESPONSE;
        
    if (*result >= 9.9e37)
        return VI_ERROR_MEASUREMENT_NOT_READY;
        
    rsNRVD_status = rsNRVD_instrStatus (instrSession);     
    return rsNRVD_status;   
}                                       

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_reset (ViSession instrSession)
{
    ViUInt32 retCnt = 0;
    ViStatus rsNRVD_status = VI_SUCCESS;

    /*  Initialize the instrument to a known state.  */
    if ((rsNRVD_status = viWrite (instrSession, "*RST", 4, &retCnt)) < 0)
        return rsNRVD_status;

    if ((rsNRVD_status = rsNRVD_defaultInstrSetup (instrSession)) < 0)  
        return rsNRVD_status;
        
    return rsNRVD_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_self_test (ViSession instrSession, ViPInt16 testResult,
                    ViChar _VI_FAR testMessage[])
{
    ViUInt32 retCnt = 0;
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViByte rdBuffer[BUFFER_SIZE];

    if ((rsNRVD_status = viWrite (instrSession, "*TST?", 5, &retCnt)) < 0)
        return rsNRVD_status;

    if ((rsNRVD_status = viRead (instrSession, rdBuffer, 256, &retCnt)) < 0)
        return rsNRVD_status;
     
    if (Scan (rdBuffer, "%s>%d[b2]", testResult) != 1)
        return VI_ERROR_INTERPRETING_RESPONSE;
        
    if (*testResult) {
        Fmt (testMessage, "Error on power-up");
        rsNRVD_status = VI_ERROR_SELF_TEST_FAILED;
    }   
    else 
        Fmt (testMessage, "Switch on test without error");
    
    return rsNRVD_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_error_query (ViSession instrSession, ViPInt32 errCode,
                    ViChar _VI_FAR errMessage[])
{
    ViUInt32 retCnt = 0;
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViByte rdBuffer[BUFFER_SIZE];

    if ((rsNRVD_status = viWrite (instrSession, ":SYST:ERR?", 10, &retCnt)) < 0)
        return rsNRVD_status;

    if ((rsNRVD_status = viRead (instrSession, rdBuffer, 256, &retCnt)) < 0)
        return rsNRVD_status;
     
    if (Scan (rdBuffer, "%s>%d[b4],\"%s[t34]", errCode, errMessage) != 2)
        return VI_ERROR_INTERPRETING_RESPONSE;

    return rsNRVD_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_error_message (ViSession instrSession, ViStatus errorCode,
                    ViChar _VI_FAR errMessage[])
{
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViInt16 i;
    static rsNRVD_tStringValPair statusDescArray[] = {
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_PARAMETER9,   "ERROR: Parameter 9 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_INSTRUMENT_ERROR, "ERROR: Instrument dependent or sensor error"},        
        {VI_ERROR_ZERO_MEAS_FAILED, "ERROR: Zero measurement has failed"},
        {VI_ERROR_MEASUREMENT_NOT_READY, "ERROR: Requested measurement is not ready"},
        {VI_ERROR_SELF_TEST_FAILED, "ERROR: Self-test has failed"},
        {VI_NULL, VI_NULL}
    };

    rsNRVD_status = viStatusDesc (instrSession, errorCode, errMessage);
    if (rsNRVD_status == VI_WARN_UNKNOWN_STATUS) {
        for (i=0; statusDescArray[i].stringName; i++) {
            if (statusDescArray[i].stringVal == errorCode) {
                strcpy (errMessage, statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        sprintf (errMessage, "Unknown Error 0x%08lX", errorCode);
        return (VI_WARN_UNKNOWN_STATUS);
    }
    
    rsNRVD_status = VI_SUCCESS;
    return rsNRVD_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_revision_query (ViSession instrSession,
                    ViChar _VI_FAR driverRev[], ViChar _VI_FAR instrRev[])
{
    ViUInt32 retCnt = 0;
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViByte rdBuffer[BUFFER_SIZE];

    if ((rsNRVD_status = viWrite (instrSession, "*IDN?", 5, &retCnt)) < 0)
        return rsNRVD_status;

    if ((rsNRVD_status = viRead (instrSession, rdBuffer, 256, &retCnt)) < 0)
        return rsNRVD_status;
     
    if (Scan (rdBuffer, "%s>%s[dt44],%s[dt44],%s[dt44],%s", instrRev) != 4)
        return VI_ERROR_INTERPRETING_RESPONSE;

    strcpy (driverRev, rsNRVD_REVISION);
    
    return rsNRVD_status;
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_writeInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCnt = 0;
    
    Fmt (wrtBuffer, "%s<%s", writeBuffer);
    if ((rsNRVD_status = viWrite (instrSession, wrtBuffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsNRVD_status;

    return rsNRVD_status;
}

/*===========================================================================*/
/* Function: Read Instrument Buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_readInstrData (ViSession instrSession, ViInt16 numBytes,
                    ViChar _VI_FAR rdBuf[], ViPInt32 bytesRead)
{
    ViStatus rsNRVD_status = VI_SUCCESS;
    
    *bytesRead = 0L;
        
    if ((rsNRVD_status = viRead (instrSession, rdBuf, numBytes, bytesRead)) < 0)
        return rsNRVD_status;

    return rsNRVD_status;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsNRVD_close (ViSession instrSession)
{
    ViSession rmSession;
    ViStatus rsNRVD_status = VI_SUCCESS;

    if ((rsNRVD_status = viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession)) < 0)
        return rsNRVD_status;
    
    rsNRVD_status = viClose (instrSession);
    viClose (rmSession);

    return rsNRVD_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean rsNRVD_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return      */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rsNRVD_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean rsNRVD_invalidViReal64Range  (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the rsNRVD_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from rsNRVD_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus rsNRVD_initCleanUp (ViSession openRMSession,
                    ViPSession openInstrSession, ViStatus currentStatus)
{
    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/*===========================================================================*/
/* Function: Status                                                          */
/* Purpose:  This function checks the event status register.                 */
/*===========================================================================*/
ViStatus rsNRVD_instrStatus (ViSession instrSession)
{
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte rdBuf[BUFFER_SIZE];
    ViInt16 esr = 0;

    if ((rsNRVD_status = viWrite (instrSession, "*ESR?", 5, &retCnt)) < 0)
        return rsNRVD_status;

    if ((rsNRVD_status = viRead (instrSession, rdBuf, 256, &retCnt)) < 0)
        return rsNRVD_status;

    if (Scan (rdBuf, "%s>%d[b2]", &esr) != 1) {
        return VI_ERROR_INTERPRETING_RESPONSE;
    }    
    if (esr & 0x38)
       rsNRVD_status = VI_ERROR_INSTRUMENT_ERROR;
    else
       rsNRVD_status = VI_SUCCESS;

    return rsNRVD_status;
}

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the rsNRVD_reset operation and by the     */
/*           rsNRVD_init function if the reset option has not been           */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus rsNRVD_defaultInstrSetup (ViSession instrSession)
{
    ViStatus rsNRVD_status = VI_SUCCESS;
    ViUInt32 retCnt;
    
    if ((rsNRVD_status = viWrite (instrSession, "*CLS", 4, &retCnt)) < 0)
        return rsNRVD_status;
    
    return rsNRVD_status;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
