
/*= Rohde & Schwarz NRVD Power Meter Include File ===========================*/

/*===========================================================================*/
/*  Please do not use global variables or arrays in the include file of      */
/*  instrument drivers that will be submitted for inclusion into the         */
/*  LabWindows Instrument Driver Library.                                    */
/*===========================================================================*/
     
#ifndef __RSNRVD_HEADER
#define __RSNRVD_HEADER

#include <vpptype.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************/
/*= Define Instrument Specific Error Codes Here =============================*/
/*****************************************************************************/
#define VI_ERROR_PARAMETER9             (_VI_ERROR+0x3FFC0009L)
#define VI_ERROR_INTERPRETING_RESPONSE  (_VI_ERROR+0x3FFC0803L)
#define VI_ERROR_INSTRUMENT_ERROR       (_VI_ERROR+0x3FFC0900L)
#define VI_ERROR_ZERO_MEAS_FAILED       (_VI_ERROR+0x3FFC0901L)
#define VI_ERROR_MEASUREMENT_NOT_READY  (_VI_ERROR+0x3FFC0902L)
#define VI_ERROR_SELF_TEST_FAILED       (_VI_ERROR+0x3FFC0903L)

/*****************************************************************************/
/*= GLOBAL USER-CALLABLE FUNCTION DECLARATIONS (Exportable Functions) =======*/
/*****************************************************************************/
ViStatus _VI_FUNC rsNRVD_init (ViRsrc resourceName, ViBoolean IDQuery,
                               ViBoolean resetDevice, ViPSession instrumentHandle);
ViStatus _VI_FUNC rsNRVD_example (ViSession instrumentHandle, ViBoolean channel,
                                  ViInt16 function, ViReal64 referenceValue,
                                  ViInt16 refValueUnit,
                                  ViReal64 _VI_FAR measuredData[],
                                  ViInt16 numberofPoints);
ViStatus _VI_FUNC rsNRVD_configSensors (ViSession instrumentHandle,
                                        ViBoolean channel, ViInt16 rangeMode,
                                        ViInt16 filter, ViReal64 manualRange,
                                        ViInt16 rangeUnit,
                                        ViReal64 attenuationonInput,
                                        ViReal64 referenceImpedance);
ViStatus _VI_FUNC rsNRVD_configFunction (ViSession instrumentHandle,
                                         ViBoolean channel, ViInt16 function,
                                         ViReal64 referenceValue,
                                         ViInt16 refValueUnit);
ViStatus _VI_FUNC rsNRVD_configMode (ViSession instrumentHandle, ViBoolean channel,
                                     ViInt16 measurementMode, ViReal64 dutyCycle);
ViStatus _VI_FUNC rsNRVD_configDisplay (ViSession instrumentHandle,
                                        ViBoolean channel,
                                        ViInt16 displayResolution,
                                        ViInt16 displayMode, ViInt16 analogBar,
                                        ViReal64 upperLimit, ViReal64 lowerLimit,
                                        ViInt16 extremeDisplay, ViReal64 SWRValue);
ViStatus _VI_FUNC rsNRVD_configFreqCorr (ViSession instrumentHandle,
                                         ViBoolean channel,
                                         ViInt16 correctionFreqInput,
                                         ViReal64 correctionFrequency,
                                         ViReal64 upperVoltage,
                                         ViReal64 upperFrequency,
                                         ViReal64 lowerVoltage,
                                         ViReal64 lowerFrequency);
ViStatus _VI_FUNC rsNRVD_configOptions (ViSession instrumentHandle,
                                        ViInt16 analogOutChannelA,
                                        ViInt16 analogOutChannelB,
                                        ViBoolean testGenerator);
ViStatus _VI_FUNC rsNRVD_zeroCorrection (ViSession instrumentHandle,
                                         ViBoolean channel, ViBoolean mode);
ViStatus _VI_FUNC rsNRVD_trigger (ViSession instrumentHandle, ViInt16 triggerMode);
ViStatus _VI_FUNC rsNRVD_operComplete (ViSession instrumentHandle,
                                       ViPBoolean complete);
ViStatus _VI_FUNC rsNRVD_measure (ViSession instrumentHandle, ViPReal64 channelA,
                                  ViPReal64 channelB);
ViStatus _VI_FUNC rsNRVD_readMeasurement (ViSession instrumentHandle,
                                          ViBoolean channel, ViPReal64 result);
ViStatus _VI_FUNC rsNRVD_reset (ViSession instrumentHandle);
ViStatus _VI_FUNC rsNRVD_self_test (ViSession instrumentHandle,
                                    ViPInt16 selfTestResult,
                                    ViChar _VI_FAR selfTestMessage[]);
ViStatus _VI_FUNC rsNRVD_error_query (ViSession instrumentHandle,
                                      ViPInt32 errorCode,
                                      ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsNRVD_error_message (ViSession instrumentHandle,
                                        ViStatus errorCode,
                                        ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsNRVD_revision_query (ViSession instrumentHandle,
                                         ViChar _VI_FAR instrumentDriverRevision[],
                                         ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC rsNRVD_writeInstrData (ViSession instrumentHandle,
                                         ViString writeBuffer);
ViStatus _VI_FUNC rsNRVD_readInstrData (ViSession instrumentHandle,
                                        ViInt16 numberBytesToRead,
                                        ViChar _VI_FAR readBuffer[],
                                        ViPInt32 numBytesRead);
ViStatus _VI_FUNC rsNRVD_close (ViSession instrumentHandle);

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif

/*****************************************************************************/
/*=== END INCLUDE FILE ======================================================*/
/*****************************************************************************/

#endif
